@@ -14,30 +14,33 @@
 # pylint: disable=invalid-name
 
 import atexit
+import datetime
 import errno
 import fnmatch
 import glob
 import grp
 import importlib.util
 import io
+import logging
 import os
 import pathlib
 import pwd
 import re
 import shutil
+import signal
 import stat
 import subprocess
 import sys
 import tempfile
+import time
 import traceback
-import typing
 import urllib.error
 import urllib.parse
 import urllib.request
 import xml.dom
 import xml.dom.minidom
 import xml.parsers.expat
-from typing import Optional
+from collections.abc import Iterable, Iterator
 
 import apport.fileutils
 import apport.logging
@@ -92,7 +95,7 @@ def _transitive_dependencies(package, depends_set):
             _transitive_dependencies(d, depends_set)
 
 
-def _read_list_files_in_directory(directory: str) -> typing.Iterator[str]:
+def _read_list_files_in_directory(directory: str) -> Iterator[str]:
     """Read every file in the directory and return each stripped line."""
     try:
         for list_file in pathlib.Path(directory).iterdir():
@@ -106,7 +109,9 @@ def _read_list_files_in_directory(directory: str) -> typing.Iterator[str]:
         pass
 
 
-def _read_proc_link(path, pid=None, dir_fd=None):
+def _read_proc_link(
+    path: str, pid: int | None = None, dir_fd: int | None = None
+) -> str:
     """Use readlink() to resolve link.
 
     Return a string representing the path to which the symbolic link points.
@@ -117,16 +122,18 @@ def _read_proc_link(path, pid=None, dir_fd=None):
     return os.readlink(f"/proc/{pid}/{path}")
 
 
-def _read_proc_file(path, pid=None, dir_fd=None):
+def _read_proc_file(
+    path: str, pid: int | None = None, dir_fd: int | None = None
+) -> str:
     """Read file content.
 
     Return its content, or return a textual error if it failed.
     """
     try:
-        if dir_fd is not None:
-            proc_file = os.open(path, os.O_RDONLY | os.O_CLOEXEC, dir_fd=dir_fd)
+        if dir_fd is None:
+            proc_file: int | str = f"/proc/{pid}/{path}"
         else:
-            proc_file = f"/proc/{pid}/{path}"
+            proc_file = os.open(path, os.O_RDONLY | os.O_CLOEXEC, dir_fd=dir_fd)
 
         with io.open(proc_file, "rb") as fd:
             return fd.read().strip().decode("UTF-8", errors="replace")
@@ -155,7 +162,7 @@ def _read_maps(proc_pid_fd):
 
 
 def _command_output(
-    command: list[str], env: Optional[dict[str, str]] = None, timeout: float = 1800
+    command: list[str], env: dict[str, str] | None = None, timeout: float = 1800
 ) -> str:
     """Run command and capture its output.
 
@@ -308,12 +315,22 @@ class _Environment(dict[str, str]):
         else:
             self["PATH"] = "(custom, no user)"
 
-    def anonymize_vars(self, keys: typing.Iterable[str]) -> None:
+    def anonymize_vars(self, keys: Iterable[str]) -> None:
         """Anonymize given environment variables if present."""
         for key in keys:
             if key in self:
                 self[key] = "<set>"
 
+    @classmethod
+    def from_string(cls, environ):
+        """Read environment variables from text representation.
+
+        The environment variables are expected to be separated by newlines.
+        """
+        return cls(
+            [entry.split("=", 1) for entry in environ.split("\n") if "=" in entry]
+        )
+
 
 #
 # Report class
@@ -327,7 +344,7 @@ class Report(problem_report.ProblemReport):
     standard debugging data.
     """
 
-    def __init__(self, problem_type="Crash", date=None):
+    def __init__(self, problem_type: str = "Crash", date: str | None = None) -> None:
         """Initialize a fresh problem report.
 
         date is the desired date/time string; if None (default), the current
@@ -337,11 +354,11 @@ class Report(problem_report.ProblemReport):
         self.pid, so that e. g. hooks can use it to collect additional data.
         """
         problem_report.ProblemReport.__init__(self, problem_type, date)
-        self.pid = None
-        self._proc_maps_cache = None
+        self.pid: int | None = None
+        self._proc_maps_cache: list[tuple[int, int, str]] | None = None
 
     @staticmethod
-    def _customized_package_suffix(package):
+    def _customized_package_suffix(package: str) -> str:
         """Return a string suitable for appending to Package/Dependencies.
 
         If package has only unmodified files, return the empty string. If not,
@@ -364,7 +381,7 @@ class Report(problem_report.ProblemReport):
 
         return suffix
 
-    def add_package(self, package):
+    def add_package(self, package: str) -> str | None:
         """Add Package: field.
 
         Determine the version of the given package (uses "(not installed") for
@@ -404,7 +421,7 @@ class Report(problem_report.ProblemReport):
             dependencies += f"{dep} {v}{self._customized_package_suffix(dep)}"
         return dependencies
 
-    def add_package_info(self, package=None):
+    def add_package_info(self, package: str | None = None) -> None:
         """Add packaging information.
 
         If package is not given, the report must have ExecutablePath.
@@ -493,7 +510,7 @@ class Report(problem_report.ProblemReport):
         """
         # Launchpad
         p = (
-            r"^https?:\/\/.*launchpad\.net\/"
+            r"^https?:\/\/.*launchpad\.net\/(?:distros/)?"
             r"((?:[^\/]+\/\+source\/)?[^\/]+)(?:.*field\.tags?=([^&]+))?"
         )
         m = re.search(p, urllib.parse.unquote(snap_contact))
@@ -510,7 +527,7 @@ class Report(problem_report.ProblemReport):
             self["SnapGitName"] = m.group(2)
             self["CrashDB"] = "snap-github"
 
-    def add_os_info(self):
+    def add_os_info(self) -> None:
         """Add operating system information.
 
         This adds:
@@ -528,14 +545,18 @@ class Report(problem_report.ProblemReport):
         if "Architecture" not in self:
             self["Architecture"] = packaging.get_system_architecture()
 
-    def add_user_info(self):
+    def add_user_info(self) -> None:
         """Add information about the user.
 
         This adds:
         - UserGroups: system groups the user is in
         """
         # Use effective uid in case privileges were dropped
-        user = pwd.getpwuid(os.geteuid())[0]
+        try:
+            user = pwd.getpwuid(os.geteuid())[0]
+        except KeyError:
+            # User not found (e.g. dynamic user in container)
+            return
         sys_gid_max = apport.fileutils.get_sys_gid_max()
         groups = [
             name
@@ -550,7 +571,7 @@ class Report(problem_report.ProblemReport):
             # UserGroups to exist
             self["UserGroups"] = "N/A"
 
-    def _check_interpreted(self):
+    def _check_interpreted(self) -> None:
         # TODO: Split into smaller functions/methods
         # pylint: disable=too-many-branches
         """Check if process is a script.
@@ -595,9 +616,9 @@ class Report(problem_report.ProblemReport):
                     self["InterpreterPath"] = self["ExecutablePath"]
                     self["ExecutablePath"] = path
                 else:
-                    self[
-                        "UnreportableReason"
-                    ] = f"Cannot determine path of python module {cmdargs[2]}"
+                    self["UnreportableReason"] = (
+                        f"Cannot determine path of python module {cmdargs[2]}"
+                    )
                 return
 
             del cmdargs[1]
@@ -634,7 +655,7 @@ class Report(problem_report.ProblemReport):
             else:
                 self["UnreportableReason"] = "Cannot determine twistd client program"
 
-    def _twistd_executable(self):
+    def _twistd_executable(self) -> str | None:
         """Determine the twistd client program from ProcCmdline."""
         args = self["ProcCmdline"].split("\0")[2:]
 
@@ -659,7 +680,7 @@ class Report(problem_report.ProblemReport):
         return None
 
     @staticmethod
-    def _python_module_path(module):
+    def _python_module_path(module: str) -> str | None:
         """Determine path of given Python module."""
         try:
             spec = importlib.util.find_spec(module)
@@ -669,9 +690,14 @@ class Report(problem_report.ProblemReport):
             return None
         return spec.origin
 
-    def add_proc_info(self, pid=None, proc_pid_fd=None, extraenv=None):
+    def add_proc_info(
+        self,
+        pid: int | str | None = None,
+        proc_pid_fd: int | None = None,
+        extraenv: Iterable[str] | None = None,
+    ) -> None:
         # TODO: Split into smaller functions/methods
-        # pylint: disable=too-many-branches,too-many-statements
+        # pylint: disable=too-many-branches
         """Add /proc/pid information.
 
         If neither pid nor self.pid are given, it defaults to the process'
@@ -692,15 +718,14 @@ class Report(problem_report.ProblemReport):
         - ProcMaps: /proc/pid/maps contents
         - ProcAttrCurrent: /proc/pid/attr/current contents, if not "unconfined"
         - CurrentDesktop: Value of $XDG_CURRENT_DESKTOP, if present
-        - _LogindSession: logind cgroup path, if present (Used for filtering
-          out crashes that happened in a session that is not running any more)
         """
+        if isinstance(pid, str):
+            pid = int(pid)
         if not proc_pid_fd:
             if not pid:
                 pid = self.pid or os.getpid()
             if not self.pid:
-                self.pid = int(pid)
-            pid = str(pid)
+                self.pid = pid
             try:
                 proc_pid_fd = os.open(
                     f"/proc/{pid}", os.O_RDONLY | os.O_PATH | os.O_DIRECTORY
@@ -739,7 +764,7 @@ class Report(problem_report.ProblemReport):
         # check if we have an interpreted program
         self._check_interpreted()
 
-        self["ExecutableTimestamp"] = str(int(os.stat(self["ExecutablePath"]).st_mtime))
+        self._add_executable_timestamp()
 
         # make ProcCmdline ASCII friendly, do shell escaping
         self["ProcCmdline"] = (
@@ -761,11 +786,15 @@ class Report(problem_report.ProblemReport):
         except OSError:
             pass
 
-        ret = self.get_logind_session(pid, proc_pid_fd)
-        if ret:
-            self["_LogindSession"] = ret[0]
+    def _add_executable_timestamp(self) -> None:
+        self["ExecutableTimestamp"] = str(int(os.stat(self["ExecutablePath"]).st_mtime))
 
-    def add_proc_environ(self, pid=None, extraenv=None, proc_pid_fd=None):
+    def add_proc_environ(
+        self,
+        pid: int | None = None,
+        extraenv: Iterable[str] | None = None,
+        proc_pid_fd: int | None = None,
+    ) -> None:
         """Add environment information.
 
         If pid is not given, it defaults to the process' current pid.
@@ -776,6 +805,23 @@ class Report(problem_report.ProblemReport):
           plus the ones mentioned in extraenv)
         - CurrentDesktop: Value of $XDG_CURRENT_DESKTOP, if present
         """
+        if not proc_pid_fd:
+            if not pid:
+                pid = os.getpid()
+            proc_pid_fd = os.open(
+                f"/proc/{pid}", os.O_RDONLY | os.O_PATH | os.O_DIRECTORY
+            )
+
+        try:
+            environ = _Environment(apport.fileutils.get_process_environ(proc_pid_fd))
+        except OSError as error:
+            self["ProcEnviron"] = f"Error: {error}"
+            return
+        self._add_environ(environ, extraenv)
+
+    def _add_environ(
+        self, environ: _Environment, extraenv: Iterable[str] | None = None
+    ) -> None:
         anonymize_vars = {"LD_LIBRARY_PATH", "LD_PRELOAD", "XDG_RUNTIME_DIR"}
         safe_vars = anonymize_vars | {
             "SHELL",
@@ -800,20 +846,6 @@ class Report(problem_report.ProblemReport):
         if extraenv:
             safe_vars |= set(extraenv)
 
-        if not proc_pid_fd:
-            if not pid:
-                pid = os.getpid()
-            pid = str(pid)
-            proc_pid_fd = os.open(
-                f"/proc/{pid}", os.O_RDONLY | os.O_PATH | os.O_DIRECTORY
-            )
-
-        try:
-            environ = _Environment(apport.fileutils.get_process_environ(proc_pid_fd))
-        except OSError as error:
-            self["ProcEnviron"] = f"Error: {error}"
-            return
-
         environ.anonymize_path()
         environ.anonymize_vars(anonymize_vars)
 
@@ -827,23 +859,22 @@ class Report(problem_report.ProblemReport):
             ]
         )
 
-    def add_kernel_crash_info(self):
+    def add_kernel_crash_info(self) -> bool:
         """Add information from kernel crash.
 
         This needs a VmCore in the Report.
         """
         if "VmCore" not in self:
             return False
-        unlink_core = False
-        ret = False
-        try:
-            if hasattr(self["VmCore"], "find"):
-                (fd, core) = tempfile.mkstemp()
-                os.write(fd, self["VmCore"])
-                os.close(fd)
-                unlink_core = True
+        with tempfile.NamedTemporaryFile(prefix="apport_vmcore_") as core:
+            if hasattr(self["VmCore"], "write"):
+                self["VmCore"].write(core)
+            else:
+                core.write(self["VmCore"])
+            core.flush()
+
             kver = self["Uname"].split()[1]
-            command = ["crash", f"/usr/lib/debug/boot/vmlinux-{kver}", core]
+            command = ["crash", f"/usr/lib/debug/boot/vmlinux-{kver}", core.name]
             try:
                 crash = subprocess.run(
                     command,
@@ -858,12 +889,11 @@ class Report(problem_report.ProblemReport):
             if ret:
                 # FIXME: split it up nicely etc
                 self["Stacktrace"] = crash.stdout
-        finally:
-            if unlink_core:
-                os.unlink(core)
         return ret
 
-    def add_gdb_info(self, rootdir=None, gdb_sandbox=None):
+    def add_gdb_info(
+        self, rootdir: str | None = None, gdb_sandbox: str | None = None
+    ) -> None:
         # TODO: Split into smaller functions/methods
         # pylint: disable=too-many-branches,too-many-locals,too-many-statements
         """Add information from gdb.
@@ -902,13 +932,6 @@ class Report(problem_report.ProblemReport):
         }
         gdb_cmd, environ = self.gdb_command(rootdir, gdb_sandbox)
         environ["HOME"] = "/nonexistent"
-        if not gdb_cmd:
-            raise FileNotFoundError(
-                errno.ENOENT,
-                os.strerror(errno.ENOENT),
-                "gdb not found in retracing env",
-            )
-
         gdb_cmd += [
             "--batch",
             # limit maximum backtrace depth (to avoid looped stacks)
@@ -1045,11 +1068,11 @@ class Report(problem_report.ProblemReport):
 
                 # handle XErrors
                 if unwinding_xerror:
-                    if fn.startswith("_X") or fn in [
-                        "handle_response",
-                        "handle_error",
-                        "XWindowEvent",
-                    ]:
+                    if (
+                        fn
+                        and fn.startswith("_X")
+                        or fn in {"handle_response", "handle_error", "XWindowEvent"}
+                    ):
                         continue
                     unwinding_xerror = False
 
@@ -1077,9 +1100,9 @@ class Report(problem_report.ProblemReport):
 
     def add_hooks_info(
         self,
-        ui: Optional[HookUI] = None,
-        package: Optional[str] = None,
-        srcpackage: Optional[str] = None,
+        ui: HookUI | None = None,
+        package: str | None = None,
+        srcpackage: str | None = None,
     ) -> bool:
         """Run hook script for collecting package specific data.
 
@@ -1099,7 +1122,7 @@ class Report(problem_report.ProblemReport):
         return ret
 
     def _add_hooks_info(
-        self, ui: HookUI, package: Optional[str], srcpackage: Optional[str]
+        self, ui: HookUI, package: str | None, srcpackage: str | None
     ) -> bool:
         # TODO: Split into smaller functions/methods
         # pylint: disable=too-many-branches
@@ -1118,7 +1141,7 @@ class Report(problem_report.ProblemReport):
         if srcpackage:
             srcpackage = srcpackage.split()[0]
             if "/" in srcpackage:
-                self["UnreportableReason"] = f"invalid SourcePackage: {package}"
+                self["UnreportableReason"] = f"invalid SourcePackage: {srcpackage}"
                 return False
 
         hook_dirs = [PACKAGE_HOOK_DIR]
@@ -1146,7 +1169,7 @@ class Report(problem_report.ProblemReport):
                 opt_path = os.path.dirname(opt_path)
 
         # common hooks
-        for hook in glob.glob(GENERAL_HOOK_DIR + "/*.py"):
+        for hook in sorted(glob.glob(GENERAL_HOOK_DIR + "/*.py")):
             if _run_hook(self, ui, hook):
                 return True
 
@@ -1166,7 +1189,7 @@ class Report(problem_report.ProblemReport):
 
         return False
 
-    def search_bug_patterns(self, url):
+    def search_bug_patterns(self, url: str | None) -> str | None:
         r"""Check bug patterns loaded from the specified url.
 
         Return bug URL on match, or None otherwise.
@@ -1258,6 +1281,16 @@ class Report(problem_report.ProblemReport):
 
         return dom
 
+    def _get_signal_name(self) -> str:
+        signal_number = int(self["Signal"])
+        signal_name = self.get("SignalName")
+        if signal_name:
+            return signal_name
+        try:
+            return signal.Signals(signal_number).name
+        except ValueError:
+            return f"signal {signal_number}"
+
     def check_ignored(self) -> bool:
         """Check if current report should not be presented.
 
@@ -1313,7 +1346,7 @@ class Report(problem_report.ProblemReport):
 
         return False
 
-    def mark_ignore(self):
+    def mark_ignore(self) -> None:
         """Ignore future crashes of this executable.
 
         Add a ignore list entry for this report to ~/.apport-ignore.xml, so
@@ -1358,7 +1391,7 @@ class Report(problem_report.ProblemReport):
 
         dom.unlink()
 
-    def has_useful_stacktrace(self):
+    def has_useful_stacktrace(self) -> bool:
         """Check whether StackTrace can be considered 'useful'.
 
         The current heuristic is to consider it useless if it either is shorter
@@ -1375,7 +1408,7 @@ class Report(problem_report.ProblemReport):
 
         return unknown_fn.count(True) <= len(unknown_fn) / 2.0
 
-    def stacktrace_top_function(self):
+    def stacktrace_top_function(self) -> str | None:
         """Return topmost function in StacktraceTop."""
         for line in self.get("StacktraceTop", "").splitlines():
             fname = line.split("(")[0].strip()
@@ -1384,10 +1417,76 @@ class Report(problem_report.ProblemReport):
 
         return None
 
-    def standard_title(self):
-        # TODO: Split into smaller functions/methods
-        # pylint: disable=too-many-branches,too-many-locals
-        # pylint: disable=too-many-return-statements,too-many-statements
+    def _get_python_exception_title(self) -> str | None:
+        trace = self["Traceback"].splitlines()
+
+        if len(trace) < 1:
+            return None
+        if len(trace) < 3:
+            return (
+                f"{os.path.basename(self['ExecutablePath'])}"
+                f" crashed with {trace[0]}"
+            )
+
+        trace_re = re.compile(r'^\s*File\s*"(\S+)".* in (.+)$')
+        i = len(trace) - 1
+        function = "unknown"
+        while i >= 0:
+            m = trace_re.match(trace[i])
+            if m:
+                module_path = m.group(1)
+                function = m.group(2)
+                break
+            i -= 1
+
+        path = os.path.basename(self["ExecutablePath"])
+        last_line = trace[-1]
+        exception = last_line.split(":")[0]
+        m = re.match(f"^{re.escape(exception)}: (.+)$", last_line)
+        if m:
+            message = m.group(1)
+        else:
+            message = None
+
+        if function == "<module>":
+            if module_path == self["ExecutablePath"]:
+                context = "__main__"
+            else:
+                # Maybe use os.path.basename?
+                context = module_path
+        else:
+            context = f"{function}()"
+
+        title = f"{path} crashed with {exception} in {context}"
+
+        if message:
+            title += f": {message}"
+
+        return title
+
+    def _get_signal_crash_title(self) -> str | None:
+        fn = self.stacktrace_top_function()
+        if fn:
+            fn = f" in {fn}()"
+        else:
+            fn = ""
+
+        arch_mismatch = ""
+        if (
+            "Architecture" in self
+            and "PackageArchitecture" in self
+            and self["Architecture"] != self["PackageArchitecture"]
+            and self["PackageArchitecture"] != "all"
+        ):
+            arch_mismatch = f" [non-native {self['PackageArchitecture']} package]"
+
+        return (
+            f"{os.path.basename(self['ExecutablePath'])}"
+            f" crashed with {self._get_signal_name()}{fn}{arch_mismatch}"
+        )
+
+    # pylint: disable-next=too-many-return-statements
+    def standard_title(self) -> str | None:
         """Create an appropriate title for a crash database entry.
 
         This contains the topmost function name from the stack trace and the
@@ -1408,86 +1507,11 @@ class Report(problem_report.ProblemReport):
                 f" {self['AssertionMessage']}"
             )
 
-        # signal crash
         if "Signal" in self and "ExecutablePath" in self and "StacktraceTop" in self:
-            signal_names = {
-                "4": "SIGILL",
-                "6": "SIGABRT",
-                "8": "SIGFPE",
-                "11": "SIGSEGV",
-                "13": "SIGPIPE",
-            }
+            return self._get_signal_crash_title()
 
-            fn = self.stacktrace_top_function()
-            if fn:
-                fn = f" in {fn}()"
-            else:
-                fn = ""
-
-            arch_mismatch = ""
-            if (
-                "Architecture" in self
-                and "PackageArchitecture" in self
-                and self["Architecture"] != self["PackageArchitecture"]
-                and self["PackageArchitecture"] != "all"
-            ):
-                arch_mismatch = f" [non-native {self['PackageArchitecture']} package]"
-
-            signal_name = signal_names.get(
-                self.get("Signal"), "signal " + self.get("Signal")
-            )
-            return (
-                f"{os.path.basename(self['ExecutablePath'])}"
-                f" crashed with {signal_name}{fn}{arch_mismatch}"
-            )
-
-        # Python exception
         if "Traceback" in self and "ExecutablePath" in self:
-            trace = self["Traceback"].splitlines()
-
-            if len(trace) < 1:
-                return None
-            if len(trace) < 3:
-                return (
-                    f"{os.path.basename(self['ExecutablePath'])}"
-                    f" crashed with {trace[0]}"
-                )
-
-            trace_re = re.compile(r'^\s*File\s*"(\S+)".* in (.+)$')
-            i = len(trace) - 1
-            function = "unknown"
-            while i >= 0:
-                m = trace_re.match(trace[i])
-                if m:
-                    module_path = m.group(1)
-                    function = m.group(2)
-                    break
-                i -= 1
-
-            path = os.path.basename(self["ExecutablePath"])
-            last_line = trace[-1]
-            exception = last_line.split(":")[0]
-            m = re.match(f"^{re.escape(exception)}: (.+)$", last_line)
-            if m:
-                message = m.group(1)
-            else:
-                message = None
-
-            if function == "<module>":
-                if module_path == self["ExecutablePath"]:
-                    context = "__main__"
-                else:
-                    # Maybe use os.path.basename?
-                    context = module_path
-            else:
-                context = f"{function}()"
-
-            title = f"{path} crashed with {exception} in {context}"
-
-            if message:
-                title += f": {message}"
-
-            return title
+            return self._get_python_exception_title()
 
         # package problem
         if self.get("ProblemType") == "Package" and "Package" in self:
@@ -1521,7 +1545,7 @@ class Report(problem_report.ProblemReport):
 
         return None
 
-    def obsolete_packages(self):
+    def obsolete_packages(self) -> list[str]:
         """Return list of obsolete packages in Package and Dependencies."""
         obsolete = []
         for line in (
@@ -1540,7 +1564,7 @@ class Report(problem_report.ProblemReport):
                 obsolete.append(pkg)
         return obsolete
 
-    def crash_signature(self):
+    def crash_signature(self) -> str | None:
         # TODO: Split into smaller functions/methods
         # pylint: disable=too-many-branches,too-many-return-statements
         # pylint: disable=too-many-statements
@@ -1566,7 +1590,7 @@ class Report(problem_report.ProblemReport):
         it exists.
         """
         if "ExecutablePath" not in self:
-            if not self["ProblemType"] in ("KernelCrash", "KernelOops"):
+            if not self["ProblemType"] in {"KernelCrash", "KernelOops"}:
                 return None
 
         # kernel crash
@@ -1659,9 +1683,9 @@ class Report(problem_report.ProblemReport):
                     parsed = re.search("^BUG: unable to handle (.*) at ", line)
                     if parsed:
                         match = parsed.group(1)
-                        assert match, (
-                            f"could not parse expected" f" problem type line: {line}"
-                        )
+                        assert (
+                            match
+                        ), f"could not parse expected problem type line: {line}"
                         parts.append(match)
 
                 if line.startswith("IP: "):
@@ -1685,7 +1709,7 @@ class Report(problem_report.ProblemReport):
         return None
 
     @staticmethod
-    def _extract_function_and_address(line):
+    def _extract_function_and_address(line: str) -> str | None:
         parsed = re.search(r"\[.*\] (.*)$", line)
         if parsed:
             match = parsed.group(1)
@@ -1694,7 +1718,7 @@ class Report(problem_report.ProblemReport):
                 return match
         return None
 
-    def crash_signature_addresses(self):
+    def crash_signature_addresses(self) -> str | None:
         """Compute heuristic duplicate signature for a signal crash.
 
         This should be used if crash_signature() fails, i. e. Stacktrace does
@@ -1754,7 +1778,7 @@ class Report(problem_report.ProblemReport):
 
         return f"{self['ExecutablePath']}:{self['Signal']}:{':'.join(stack)}"
 
-    def anonymize(self):
+    def anonymize(self) -> None:
         """Remove user identifying strings from the report.
 
         This particularly removes the user name, host name, and IPs
@@ -1799,7 +1823,7 @@ class Report(problem_report.ProblemReport):
             if (
                 is_proc_field
                 or "Stacktrace" in k
-                or k in ["Traceback", "PythonArgs", "Title", "JournalErrors"]
+                or k in {"Traceback", "PythonArgs", "Title", "JournalErrors"}
             ):
                 if not hasattr(self[k], "isspace"):
                     continue
@@ -1811,7 +1835,9 @@ class Report(problem_report.ProblemReport):
                     else:
                         self[k] = pattern.sub(repl, self[k])
 
-    def gdb_command(self, sandbox, gdb_sandbox=None):
+    def gdb_command(
+        self, sandbox: str | None, gdb_sandbox: str | None = None
+    ) -> tuple[list[str], dict[str, str]]:
         # TODO: Split into smaller functions/methods
         # pylint: disable=too-many-branches,too-many-locals
         """Build gdb command for this report.
@@ -1841,9 +1867,14 @@ class Report(problem_report.ProblemReport):
         )
         gdb_path = _which_extrapath("gdb", gdb_sandbox_bin)
         if not gdb_path:
-            return "", ""
+            raise FileNotFoundError(
+                errno.ENOENT,
+                os.strerror(errno.ENOENT),
+                "gdb not found in retracing env",
+            )
+
         command = [gdb_path]
-        environ = {}
+        environ: dict[str, str] = {}
 
         if not same_arch:
             # check if we have gdb-multiarch
@@ -1883,12 +1914,11 @@ class Report(problem_report.ProblemReport):
                 # env settings need to be modified for gdb in a sandbox
                 environ |= {
                     "LD_LIBRARY_PATH": ld_lib_path,
+                    "PATH": ld_lib_path,
                     "PYTHONHOME": pyhome,
-                    "GCONV_PATH": (f"{gdb_sandbox}/usr/lib/{native_multiarch}/gconv"),
+                    "GCONV_PATH": f"{gdb_sandbox}/usr/lib/{native_multiarch}/gconv",
                 }
-                command.insert(
-                    0, f"{gdb_sandbox}" f"/lib/{native_multiarch}/ld-linux-x86-64.so.2"
-                )
+                command[:0] = ["ld-linux-x86-64.so.2"]
                 command += ["--ex", f"set data-directory {gdb_sandbox}/usr/share/gdb"]
             if not os.path.exists(sandbox + executable):
                 if executable.startswith("/usr"):
@@ -1904,7 +1934,7 @@ class Report(problem_report.ProblemReport):
                 atexit.register(os.unlink, core)
                 os.write(fd, self["CoreDump"])
                 os.close(fd)
-            elif hasattr(self["CoreDump"], "gzipvalue"):
+            elif isinstance(self["CoreDump"], problem_report.CompressedValue):
                 (fd, core) = tempfile.mkstemp(prefix="apport_core_")
                 atexit.register(os.unlink, core)
                 os.close(fd)
@@ -1940,7 +1970,7 @@ class Report(problem_report.ProblemReport):
 
         return None
 
-    def _build_proc_maps_cache(self):
+    def _build_proc_maps_cache(self) -> None:
         """Generate self._proc_maps_cache from ProcMaps field.
 
         This only gets done once.
@@ -1973,35 +2003,84 @@ class Report(problem_report.ProblemReport):
                 (int(m.group(1), 16), int(m.group(2), 16), m.group(3))
             )
 
-    @staticmethod
-    def get_logind_session(pid=None, proc_pid_fd=None):
-        """Get logind session path and start time.
+    def _add_str_from_coredump(
+        self, coredump: dict[str, object], coredump_key: str, key: str
+    ) -> None:
+        value = coredump.get(coredump_key)
+        assert isinstance(value, str)
+        self[key] = value.rstrip()
 
-        Return (session_id, session_start_timestamp) if process is in a logind
-        session, or None otherwise.
+    def _add_coredump_from_systemd_coredump(self, coredump: dict[str, object]) -> None:
+        dump = coredump.get("COREDUMP")
+        if dump:
+            assert isinstance(dump, bytes)
+            self["CoreDump"] = dump
+            return
+
+        filename = coredump.get("COREDUMP_FILENAME")
+        if filename:
+            assert isinstance(filename, str)
+            try:
+                self["CoreDump"] = problem_report.CompressedFile(filename)
+            except FileNotFoundError:
+                logging.getLogger(__name__).warning(
+                    "Ignoring COREDUMP_FILENAME '%s' because it does not exist.",
+                    filename,
+                )
+
+    @classmethod
+    def from_systemd_coredump(cls, coredump):
+        """Convert the givin systemd coredump into a problem report.
+
+        The following keys are not used:
+         * COREDUMP_CGROUP (str)
+         * COREDUMP_COMM (str)
+         * COREDUMP_CONTAINER_CMDLINE (str)
+         * COREDUMP_GID (int)
+         * COREDUMP_HOSTNAME (str)
+         * COREDUMP_OPEN_FDS (str)
+         * COREDUMP_OWNER_UID (str)
+         * COREDUMP_PACKAGE_JSON (str)
+         * COREDUMP_PACKAGE_NAME
+         * COREDUMP_PACKAGE_VERSION
+         * COREDUMP_PROC_AUXV (bytes)
+         * COREDUMP_PROC_CGROUP (str)
+         * COREDUMP_PROC_LIMITS (str)
+         * COREDUMP_PROC_MOUNTINFO (str)
+         * COREDUMP_RLIMIT (str)
+         * COREDUMP_ROOT (str)
+         * COREDUMP_SESSION
+         * COREDUMP_SLICE (str)
+         * COREDUMP_UID (int)
+         * COREDUMP_UNIT (str)
+         * COREDUMP_USER_UNIT (str)
+         * MESSAGE (str)
         """
-        if proc_pid_fd is not None:
-            cgroup_file = os.open("cgroup", os.O_RDONLY, dir_fd=proc_pid_fd)
-        else:
-            cgroup_file = f"/proc/{pid}/cgroup"
+        date = coredump.get("COREDUMP_TIMESTAMP")
+        assert isinstance(date, datetime.datetime)
+        report = cls(date=time.asctime(date.timetuple()))
 
-        # determine cgroup
-        try:
-            with io.open(cgroup_file, encoding="utf-8") as f:
-                for line in f:
-                    line = line.strip()
-                    if (
-                        "name=systemd:" in line
-                        and line.endswith(".scope")
-                        and "/session-" in line
-                    ):
-                        my_session = line.split("/session-", 1)[1][:-6]
-                        break
-                else:
-                    return None
-            # determine session creation time
-            session_start_time = os.stat("/run/systemd/sessions/" + my_session).st_mtime
-        except OSError:
-            return None
+        pid = coredump.get("COREDUMP_PID")
+        assert isinstance(pid, int)
+        report.pid = pid
 
-        return (my_session, session_start_time)
+        signal_number = coredump.get("COREDUMP_SIGNAL")
+        assert isinstance(signal_number, int)
+        report["Signal"] = str(signal_number)
+
+        report._add_str_from_coredump(coredump, "COREDUMP_SIGNAL_NAME", "SignalName")
+        report._add_str_from_coredump(coredump, "COREDUMP_CMDLINE", "ProcCmdline")
+        report._add_str_from_coredump(coredump, "COREDUMP_EXE", "ExecutablePath")
+        report._add_str_from_coredump(coredump, "COREDUMP_CWD", "ProcCwd")
+        report._add_str_from_coredump(coredump, "COREDUMP_PROC_MAPS", "ProcMaps")
+        report._add_str_from_coredump(coredump, "COREDUMP_PROC_STATUS", "ProcStatus")
+
+        environ = coredump.get("COREDUMP_ENVIRON")
+        assert isinstance(environ, str)
+        report._add_environ(_Environment.from_string(environ))
+
+        report._add_executable_timestamp()
+        if report.get("COREDUMP_TRUNCATED") != "1":
+            report._add_coredump_from_systemd_coredump(coredump)
+
+        return report
