@@ -29,6 +29,11 @@ def get_data_directory(local_path: str | None = None) -> pathlib.Path:
     return pathlib.Path("/usr/share/apport")
 
 
+def get_test_data_directory() -> pathlib.Path:
+    """Return absolute path for apport's tests/data directory."""
+    return pathlib.Path(__file__).absolute().parent / "data"
+
+
 def is_local_source_directory() -> bool:
     """Return True if the current working directory is the source directory.
 
@@ -38,7 +43,7 @@ def is_local_source_directory() -> bool:
     return os.path.isdir("tests") and os.path.exists("setup.py")
 
 
-def local_test_environment() -> Mapping[str, str]:
+def local_test_environment() -> dict[str, str]:
     """Return needed environment variables when running tests locally."""
     if not is_local_source_directory():
         return {}
@@ -50,7 +55,7 @@ def local_test_environment() -> Mapping[str, str]:
     }
 
 
-def patch_data_dir(report: Any) -> Mapping[str, str] | None:
+def patch_data_dir(report: Any) -> dict[str, str] | None:
     """Patch APPORT_DATA_DIR in apport.report for local tests."""
     if not is_local_source_directory():
         return None
