@@ -9,6 +9,7 @@ import io
 import os
 import pathlib
 import re
+import subprocess
 import tempfile
 import textwrap
 import unittest
@@ -17,6 +18,7 @@ from unittest.mock import MagicMock
 
 import apport.packaging
 import apport.report
+import problem_report
 
 DIVIDE_BY_ZERO_PROC_MAPS = f"""\
 558164bd3000-558164bd4000 r--p 00000000 fc:00 44306916 \
@@ -1111,19 +1113,12 @@ CR2: 00000000ffffb4ff
         """Methods get along with non-ASCII data."""
         # fake os.uname() into reporting a non-ASCII name
         uname = os.uname()
-        uname = (
-            uname[0],
-            b"t\xe2\x99\xaax".decode("UTF-8"),
-            uname[2],
-            uname[3],
-            uname[4],
+        uname_mock = os.uname_result(
+            (uname[0], b"t\xe2\x99\xaax".decode("UTF-8"), uname[2], uname[3], uname[4])
         )
-        orig_uname = os.uname
-        os.uname = lambda: uname
-
-        try:
+        with unittest.mock.patch("os.uname", return_value=uname_mock):
             pr = apport.report.Report()
-            utf8_val = b"\xc3\xa4 " + uname[1].encode("UTF-8") + b" \xe2\x99\xa5 "
+            utf8_val = b"\xc3\xa4 " + uname_mock[1].encode("UTF-8") + b" \xe2\x99\xa5 "
             pr["ProcUnicodeValue"] = utf8_val.decode("UTF-8")
             pr["ProcByteArrayValue"] = utf8_val
 
@@ -1132,8 +1127,6 @@ CR2: 00000000ffffb4ff
             exp_utf8 = b"\xc3\xa4 hostname \xe2\x99\xa5 "
             self.assertEqual(pr["ProcUnicodeValue"], exp_utf8.decode("UTF-8"))
             self.assertEqual(pr["ProcByteArrayValue"], exp_utf8)
-        finally:
-            os.uname = orig_uname
 
     def test_address_to_offset(self):
         """_address_to_offset()"""
@@ -1320,16 +1313,14 @@ No symbol table info available.
         self.assertIsNone(pr.crash_signature_addresses())
 
     @staticmethod
-    def test_missing_uid():
+    @unittest.mock.patch("os.geteuid")
+    def test_missing_uid(geteuid_mock: MagicMock) -> None:
         """check_ignored() works for removed user"""
-        orig_getuid = os.getuid
-        os.getuid = lambda: 123456789
-        try:
-            pr = apport.report.Report()
-            pr["ExecutablePath"] = "/bin/bash"
-            pr.check_ignored()
-        finally:
-            os.getuid = orig_getuid
+        geteuid_mock.return_value = 123456789
+        pr = apport.report.Report()
+        pr["ExecutablePath"] = "/bin/bash"
+        pr.check_ignored()
+        geteuid_mock.assert_called_once_with()
 
     def test_suspend_resume(self):
         pr = apport.report.Report()
@@ -1369,6 +1360,16 @@ No symbol table info available.
         self.assertEqual("ubuntu/+source/chromium-browser", pr["SnapSource"])
         self.assertEqual("snap", pr["SnapTags"])
 
+    def test_add_snap_contact_launchpad_distro(self) -> None:
+        """add_snap_contact_info() with https://launchpad.net/distros/ contact"""
+        report = apport.report.Report()
+        snap_contact = "https://launchpad.net/distros/ubuntu/+source/thunderbird"
+
+        report.add_snap_contact_info(snap_contact)
+
+        self.assertEqual(set(report.keys()), {"ProblemType", "Date", "SnapSource"})
+        self.assertEqual("ubuntu/+source/thunderbird", report["SnapSource"])
+
     def test_add_snap_contact_info_github(self):
         """add_snap_contact_info()
 
@@ -1468,3 +1469,61 @@ No symbol table info available.
         report.write(report_output)
         self.assertEqual(report_output.getvalue().decode(), DIVIDE_BY_ZERO_REPORT)
         stat_mock.assert_called_once_with("/usr/bin/divide-by-zero")
+
+    def test_add_kernel_crash_info_no_vmcore(self) -> None:
+        """add_kernel_crash_info() on a non-kernel crash."""
+        report = apport.report.Report()
+        self.assertFalse(report.add_kernel_crash_info())
+
+    @unittest.mock.patch("subprocess.run")
+    def test_add_kernel_crash_info(self, run_mock: MagicMock) -> None:
+        """add_kernel_crash_info() on a fake vmcore."""
+        run_mock.return_value = subprocess.CompletedProcess(
+            args=MagicMock(), returncode=0, stdout=b"kernel stack trace", stderr=b""
+        )
+        report = apport.report.Report("KernelCrash")
+        report["VmCore"] = problem_report.CompressedValue(b"\x01" * 100, name="VmCore")
+        # add_os_info() keys Architecture, DistroRelease, Uname from Ubuntu 24.04
+        report["Architecture"] = "amd64"
+        report["DistroRelease"] = "Ubuntu 24.04"
+        report["Uname"] = "Linux 6.8.0-31-generic x86_64"
+
+        self.assertTrue(report.add_kernel_crash_info())
+
+        self.assertEqual(report.get("Stacktrace"), b"kernel stack trace")
+        run_mock.assert_called_once()
+        called_cmd = run_mock.call_args[0][0]
+        self.assertEqual(
+            called_cmd[:-1], ["crash", "/usr/lib/debug/boot/vmlinux-6.8.0-31-generic"]
+        )
+        tmpfile = called_cmd[-1]
+        self.assertRegex(tmpfile, r"^/.*/apport_vmcore_\w{8}$")
+        self.assertFalse(pathlib.Path(tmpfile).exists())
+
+    @unittest.mock.patch("subprocess.run")
+    def test_add_kernel_crash_info_fail(self, run_mock: MagicMock) -> None:
+        """add_kernel_crash_info() on a fake vmcore where crash fails."""
+        run_mock.return_value = subprocess.CompletedProcess(
+            args=MagicMock(),
+            returncode=1,
+            stdout=b"read_maps: unable to read header from $vmcore, errno = 0",
+            stderr=b"",
+        )
+        report = apport.report.Report("KernelCrash")
+        report["VmCore"] = b"\x01" * 100
+        # add_os_info() keys Architecture, DistroRelease, Uname from Ubuntu 24.04
+        report["Architecture"] = "amd64"
+        report["DistroRelease"] = "Ubuntu 24.04"
+        report["Uname"] = "Linux 6.8.0-31-generic x86_64"
+
+        self.assertFalse(report.add_kernel_crash_info())
+
+        self.assertNotIn("Stacktrace", report)
+        run_mock.assert_called_once()
+        called_cmd = run_mock.call_args[0][0]
+        self.assertEqual(
+            called_cmd[:-1], ["crash", "/usr/lib/debug/boot/vmlinux-6.8.0-31-generic"]
+        )
+        tmpfile = called_cmd[-1]
+        self.assertRegex(tmpfile, r"^/.*/apport_vmcore_\w{8}$")
+        self.assertFalse(pathlib.Path(tmpfile).exists())
